
Box ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Box")
docDependsOn("Vector")
docDescription("An object for fast operations on 2 to 4 dimensional rectangles.")


    docSlot("origin", 
    "Returns the point object for the origin of the box.")
    

    docSlot("size", 
    "Returns the point object for the size of the box. ")
    

    docSlot("width", 
    "Same as; size x")
    

    docSlot("height", 
    "Same as; size y")
    

    docSlot("depth", 
    "Same as; size z")
    

    docSlot("set(origin, size)", 
    "Copies the values in origin and size to set the box's origin and size.")
    

    docSlot("setOrigin(aPoint)", 
    "Copies the values in aPoint to the box's origin point. ")
    

    docSlot("setSize(aPoint)", 
    "Copies the values in aPoint to the box's size point.  ")
    

    docSlot("copy(aBox)", 
    "Copies the values of aBox to the receiver.")
    

    docSlot("print", 
    "Prints a string representation of the receiver to the standard output.")
    

    docSlot("Union(aBox)", 
    "Returns a new box containing the 2d union of the receiver and aBox.")
    

    docSlot("containsPoint(aPoint)", 
    "Returns self if aPoint is within the receiver's bounds, Nil otherwise.")
    
)

Vector ioDoc(
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
docObject("Vector")
docDescription("""An object for fast operations on 1 dimensional arrays of single-precision floating point values. Vector processor acceleration is used for certain operations on some platforms.

<pre>
    Io> v1 := vector(1, 2, 3)                    
    ==> vector(1, 2, 3)
    Io> v1 *= 2
    ==> vector(2, 4, 6)
    Io> v1 += 1
    ==> vector(3, 5, 7)
    Io> v1 -= 1
    ==> vector(2, 4, 6)
    Io> v1 /= 2
    ==> vector(1, 2, 3)
    Io> v1 += v1
    ==> vector(2, 4, 6)
</pre>""")


    docSlot("size", 
    "Returns the number of elements in the receiver. ")
    

    docSlot("setSize(aNumber)", 
    "Sets the number of elements in the receiver. Returns self.")
    

    docSlot("asBuffer", 
    "Returns the elements of the vector as a buffer of floating 
    point values in the current architecture's byte order and float format.")
    

    docSlot("== aVector", 
    "Returns self if all elements of aVector are equal to 
    those of the receiver. Returns Nil otherwise.")
    

    docSlot("copy(aVectorOrBuffer)", 
    "Sets the size of the receiver to match that of aVector and copies 
    all elements of aVector to the receiver. If the argument is a Buffer, 
    it will copy it's data, leaving any remaining bytes set to zero.")
    

    docSlot("subarray(startIndex, length)", 
    "Returns a new Vector containing the elements from startIndex to startIndex + length.")
    

    docSlot("at(indexNumber)", 
    "Returns a Number containing the value of the receivers element at indexNumber.")
    

    docSlot("atPut(indexNumber, numberValue)", 
    "Puts numberValue into the element at indexNumber. 
    Expands the size of the vector if needed.")
    

    docSlot("+=(aValue)", 
    "If aValue is a Number, an in-place scalar addition is performed on the receiver. 
    If aValue is a vector, an in-place vector addition is performed on the receiver. Otherwise, a 'Vector +=' exception is raised. Returns self.")
    

    docSlot("+(aValue)", "Same as: receiver clone +=(aValue)")
    

    docSlot("-=(aValue)", 
    "If aValue is a Number, an in-place scalar subtraction is performed on the receiver. 
    If aValue is a vector, an in-place vector subtraction is performed on the receiver. Otherwise, a Vector -= exception is raised. Returns self.")
    

    docSlot("-(aValue)", "Same as: receiver clone -=(aValue)")
    

    docSlot("*=(aValue)", 
    "If aValue is a Number, each element of the receiver is replaced by the product of it's value with aValue. If aValue is a vector, each element of the receiver is replaced by the product it's value with the element at the same index in the aValue vector. Returns self.")
    

    docSlot("*(aValue)", "Same as: receiver clone *=(aValue)")
    

    docSlot("*=(aValue)", 
    "If aValue is a Number, each element of the receiver is replaced by the it's value divided by aValue. If aValue is a vector, each element of the receiver is replaced by the product it's value divided by the element at the same index in the aValue vector. Returns self.")
    

    docSlot("/(aValue)", "Same as: receiver clone /=(aValue)")
    

    docSlot("dotProduct(aVector)", 
    "Same as '(receiver + aVector) sum', although the implementation is much faster.")
    

    docSlot("sum", "Returns the sum of all elements of the receiver.")
    

    docSlot("min", 
    "Returns the element of the vector with the lowest value.")
    

    docSlot("max", 
    "Returns the element of the vector with the greatest value.")
    

    docSlot("mean", 
    "Same as 'receiver sum / receiver size'.")
    

    docSlot("squares", 
    "Each element of the vector is squared. Returns self.")
    

    docSlot("normalize", 
    "Same as 'receiver /= receiver sum', but with a faster implementation. Returns self.")
    

    docSlot("absolute", 
    "Each element of the receiver is replaced with it's absolute value. Returns self.")
    

    docSlot("log", 
    "Each element of the receiver is replaced with it's natural log value. Returns self.")
    

    docSlot("log10", 
    "Each element of the receiver is replaced with it's log base 10 value. Returns self.")
    

    docSlot("pow(aNumber)", 
    "Each element of the receiver is replaced with it's value taken to the power of aNumber. Returns self.")
    

    docSlot("random(minNumber, maxNumber)", 
    "Each element of the receiver is replaced with random value between minNumber and maxNumber. Returns self.")
    

    docSlot("sort", 
    "The elements of the receiver are sorted from smallest to greatest. Returns self. ")
    

    docSlot("reverseSort", 
    "The elements of the receiver are sorted from greatest to smallest. Returns self. ")
    

    docSlot("meanSquare", 
    "Same as 'receiver clone square sum / receiver size'.")
    

    docSlot("rootMeanSquare", 
    "Same as 'receiver meanSquare squareRoot' ")
    

    docSlot("setMin(aNumber)", 
    "Sets all elements of the receiver that are less than aNumber to aNumber. Returns self.")
    

    docSlot("setMax(aValue)", 
    "Sets all elements of the receiver that are greater than aNumber to aNumber. Returns self.")
    

    docSlot("set(value1, value2, ...)", 
    "Sets the elements of the array to the value of the arguments. Returns self.")
    

    docSlot("setAll(aNumber)", 
    "Sets all elements of the receiver to aNumber.")
    

    docSlot("sin", 
    "Sets all elements to the sin of their value. Returns self.")
    

    docSlot("x", "Same as 'receiver at(0)' ")
    

    docSlot("width", "Same as 'receiver at(0)'")
    

    docSlot("y", "Same as 'receiver at(1)'")
    

    docSlot("height", "Same as 'receiver at(1)'")
    

    docSlot("z", "Same as 'receiver at(2)'")
    

    docSlot("depth", "Same as 'receiver at(2)'")
    

    docSlot("w", "Same as 'receiver at(3)'")
    

    docSlot("setX(aValue)", 
    "Same as 'receiver atPut(0, aValue)'")
    

    docSlot("setWidth(aValue)", "Same as 'receiver atPut(0, aValue)'")
    

    docSlot("setY(aValue)", "Same as 'receiver atPut(0, aValue)'")
    

    docSlot("setHeight(aValue)", "Same as 'receiver atPut(0, aValue)'")
    

    docSlot("setZ(aValue)", "Same as: receiver atPut(0, aValue)")
    

    docSlot("setDepth(aValue)", "Same as: receiver atPut(0, aValue)")
    

    docSlot("setW(aValue)", "Same as: receiver atPut(3, aValue)")
    

    docSlot("negate", "Multiplies each element by -1. Returns self.")
    

    docSlot("<=(aValue)", 
    "If aValue is a Vector, self is returned if each element of the receiver 
    is less than or equal to the corresponding element in aValue or Nil is returned otherwise. 
    If aValue is a Number, then self is returned if all elements of the receiver are less 
    than aValue or Nil otherwise. If aValue is niether a Number or a Vector, an exception is raised. ")
    

    docSlot(">=(aValue)", 
    "If aValue is a Vector, self is returned if each element of the receiver is greater than or equal to the corresponding element in aValue or Nil is returned otherwise. If aValue is a Number, then self is returned if all elements of the receiver are greater than aValue or Nil otherwise. If aValue is niether a Number or a Vector, an exception is raised. ")
    

    docSlot("cross(aVector)", 
    "Sets the receiver to the cross product of itself with aVector. Returns self.")
    

    docSlot("ceil", 
    "Applies the ceil operator to each element of the receiver. Return self. ")
    

    docSlot("floor", 
    "Applies the floor operator to each element of the receiver. Return self. ")
    

    docSlot("distanceTo(aVector)", 
    "Returns the distance from the receiver to aVector. ")
    

    docSlot("Min(aVector)", 
    "Sets each element of the receiver to be the lesser of 
    itself and the corresponding element of aVector.")
    

    docSlot("Max(aVector)", 
    "Sets each element of the receiver to be the greater of 
    itself and the corresponding element of aVector.")
    

    docSlot("zero", "Sets each element of the receiver to zero.")
    

    docSlot("isZero", 
    "Returns self if all elements of the receiver are zero, Nil otherwise.")
    

    docSlot("print", 
    "Prints a human readable representation of the receiver to standard output.")
    
)
