/*
docCopyright("Steve Dekorte", 2002)
docLicense("BSD revised")
*/

#ifndef IOREGULAREXPRESSION_DEFINED
#define IOREGULAREXPRESSION_DEFINED 1

#include "IoObject.h"
#include "IoSeq.h"
#include "pcre.h"

#define ISREGULAREXPRESSION(self) IoObject_hasCloneFunc_(self, (TagCloneFunc *)IoRegex_rawClone)

typedef IoObject IoRegex;

typedef struct
{
    pcre *re;
    pcre_extra *pe;
    int *outputVector;
    int outputVectorSize;
    IoSymbol *pattern;
    IoSymbol *string;
    int lastIndex;
    int compileOptions;
    int execOptions;
    int matchGroupSize;
} IoRegexData;

IoRegex *IoRegex_rawClone(IoRegex *self);
IoRegex *IoRegex_proto(void *state);
IoRegex *IoRegex_new(void *state);

void IoRegex_free(IoRegex *self);
void IoRegex_mark(IoRegex *self);

/* ----------------------------------------------------------- */
IoObject *IoRegex_setPattern(IoRegex *self, IoObject *locals, IoMessage *m);
void IoRegex_justUpdatePattern(IoRegex *self, IoMessage *m);
IoObject *IoRegex_pattern(IoRegex *self, IoObject *locals, IoMessage *m);

IoObject *IoRegex_setString(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_string(IoRegex *self, IoObject *locals, IoMessage *m);

IoObject *IoRegex_firstMatchRange(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_nextMatchRange(IoRegex *self, IoObject *locals, IoMessage *m);

IoObject *IoRegex_firstMatch(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_nextMatch(IoRegex *self, IoObject *locals, IoMessage *m);

IoObject *IoRegex_allMatches(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_hasMatch(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_version(IoRegex *self, IoObject *locals, IoMessage *m);

/* --- compileOptions ----------------------------------- */

IoObject *IoRegex_caselessOn(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_caselessOff(IoRegex *self, IoObject *locals, IoMessage *m);

IoObject *IoRegex_dollarEndOnlyOn(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_dollarEndOnlyOff(IoRegex *self, IoObject *locals, IoMessage *m);

IoObject *IoRegex_dotAllOn(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_dotAllOff(IoRegex *self, IoObject *locals, IoMessage *m);

IoObject *IoRegex_extendedOn(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_extendedOff(IoRegex *self, IoObject *locals, IoMessage *m);

IoObject *IoRegex_multilineOn(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_multilineOff(IoRegex *self, IoObject *locals, IoMessage *m);

IoObject *IoRegex_greedyOn(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_greedyOff(IoRegex *self, IoObject *locals, IoMessage *m);

IoObject *IoRegex_UTF8On(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_UTF8Off(IoRegex *self, IoObject *locals, IoMessage *m);

/* --- execOptions ----------------------------------- */

IoObject *IoRegex_matchEmptyOn(IoRegex *self, IoObject *locals, IoMessage *m);
IoObject *IoRegex_matchEmptyOff(IoRegex *self, IoObject *locals, IoMessage *m);

#endif
