
Coroutine do(
	type := "Coroutine"
	newSlot("exception", nil)
	newSlot("parentCoroutine", nil)
	newSlot("runTarget", nil)
	newSlot("runLocals", nil)
	newSlot("runMessage", nil)
	newSlot("result", nil)
	newSlot("label", nil)
	label := method("Coroutine_" .. self uniqueId)
	
	typeId := method(self type .. "_0x" .. self uniqueId asString toBase(16) .. "-" .. self label)

	callStack := method(
		ioStack select(v, getSlot("v") hasSlot("thisMessage")) reverse
	)

	stackFrameDescription := method(frame,
		m := frame thisMessage 
		m ifNil(m := Object clone do(name := "[no message]"; lineNumber := "?"; label := "?"))
		s := frame self type .. " " .. m name
		s .. " " repeated(s size - 35) .. m label lastPathComponent .. " line " .. m lineNumber
	)

	backTraceString := method(
		buf := Sequence clone
		if(exception, buf appendSeq("\nException: ", exception error, "\n\n"))
		buf appendSeq("  ", self typeId)
		
		if(callStack size > 0) then(
			buf appendSeq("\n  -------------------\n")

			if(exception,
				tmp := Object clone
				tmp thisMessage := exception caughtMessage
				tmp type := "?"
				tmp self := tmp
				buf appendSeq("  ", stackFrameDescription(tmp), "\n")
			)

			callStack foreach(v, buf appendSeq("  ", stackFrameDescription(v), "\n"))
			buf appendSeq("\n")
		) else (buf appendSeq(" empty\n"))

		//if (parentCoroutine, buf appendSeq(parentCoroutine backTraceString))
		buf
	)
	
	showStack := method(write(backTraceString))

	resumeParentCoroutine := method(
		//writeln(self typeId, " resumeParentCoroutine ", parentCoroutine typeId)
		if(parentCoroutine,
			parentCoroutine resume
			parentCoroutine scheduleNext
		)
		self pause
	)

	main := method(
		setResult(runTarget doMessage(runMessage, runLocals))
		resumeParentCoroutine
		self pause
	)

	raiseException := method(e, 
		self setException(e)
		resumeParentCoroutine
	)
)

Message do(
	codeOfLength := method(length,
		c := self code
		if (c size < length, c, c slice(0, length) .. "...") asMutable replaceSeq("\n", ";")
	)

	asStackEntry := method(
		label := label lastPathComponent fileName
		label .. " " repeated(label size - 18) .. lineNumber .. " " repeated(lineNumber asString size - 6)  .. name
	)
)

Object try := method(
	//writeln("try(", thisMessage argAt(0) code, ") ----------------------")
	currentCoro := Scheduler currentCoroutine
	coro := Coroutine clone
	coro setParentCoroutine(currentCoro) 
	coro setRunTarget(sender) 
	coro setRunLocals(sender)
	coro setRunMessage(thisMessage argAt(0)) 
	coro run
	coro scheduleNext
	currentCoro pause
	if(coro exception, coro exception, nil)
)

Object coroDo := method(
	currentCoro := Scheduler currentCoroutine
	coro := Coroutine clone
	coro setRunTarget(self) 
	coro setRunLocals(sender)
	coro setRunMessage(thisMessage argAt(0)) 
	coro run
	coro scheduleNext
	coro
)

nil do(
	catch := nil
	pass := nil
)

Protos Exception do(
	type := "Exception"
	newSlot("error", nil)
	newSlot("coroutine", nil)
	newSlot("caughtMessage", nil)
	
	raise := method(error,
		currentCoro := Scheduler currentCoroutine
		e := Exception clone setError(error) setCoroutine(currentCoro)
		currentCoro raiseException(e)
	)
	
	catch := method(exceptionProto,
		if (self isKindOf(exceptionProto),
			sender doMessage(thisMessage argAt(1))
		)
		self
	)

	pass := method(
		Scheduler currentCoroutine raiseException(self)
	) 

	showStack := method(coroutine showStack)
)
