UnitTest := Object clone do(
	type  := "UnitTest"
	setUp := method(nil)
	tearDown := method(nil)
	init := method(self exceptions := List clone)
	testSlotNames := method(self slotNames select(n, n beginsWithSeq("test")) sort)
	
	testCount := method(testSlotNames size)
	
	run := method(
		testSlotNames foreach(n, 
			//write("  ", n);
			self setUp
			e := try(self doString(n))
			if(e, 
				write("    ", n, " - failed")
				exceptions append(e)
				write("\n")
				writeln(e showStack)
				//System exit
			)
			//write("--\n")
			self tearDown
		)
	)
    
    fail := method(Exception raise("fail"))
    
	assertEquals := method(a, b, m,
		m ifNil(m = thisMessage)
		d :=  m argAt(0) code .. " != " .. thisMessage argAt(1) code
		if(a != b, Exception raise("[" .. d .. "] [" .. a .. " != " .. b .. "]"))
	)

	assertNotEquals := method(a, b, if(a == b, Exception raise(a .. " == " .. b)))

    assertSame    := method(a, b, assertEquals(a uniqueId, b uniqueId, thisMessage))
    assertNotSame := method(a, b, assertNotEquals(a uniqueId, b uniqueId, thisMessage))
    assertNil     := method(a, assertEquals(a, nil, thisMessage))
    assertNotNil  := method(a, assertNotEquals(a, nil, thisMessage))
    assertTrue    := method(a, assertEquals(a, true, thisMessage))
    assertFalse   := method(a, assertEquals(a, false, thisMessage))

	assertRaisesException := method(
		e := try(
			sender doMessage(thisMessage argAt(0))
			writeln("Should have raised Exception")
			Exception raise("Should have raised Exception")
		) 
	)
    
	knownBug := method(writeln("  [known bug: ", thisMessage argAt(0) code, "]"))
  
	assertEqualsWithinDelta := method(expected, actual, delta,
		if(((expected - actual) abs > delta),
			Exception raise("expected " .. expected .. " but was " .. actual .. " (allowed delta: " .. delta .. ")")
		)
	)
)

TestSuite := Object clone do(
	name := method(
		path asMutable pathComponent lastPathComponent
	)
	newSlot("path", ".")
	run := method(
		writeln("\n", name)
		unitTestFiles := Directory with(launchPath) files select(f, f name endsWithSeq("Test.io"))
		exceptions := List clone
		testCount := 0
		
		unitTestFiles foreach(f, 
			writeln("  ", f name fileName)
			test := Lobby doString(f contents, f path)
			test run
			testCount = testCount + test testCount
			exceptions appendSeq(test exceptions)
		)
		
		writeln("  ---------------")
		Collector collect
		writeln("  ", testCount, " tests, ", exceptions size, " failures\n")
	)
)